note
	description: "Example showing use of agents & tuples"
	author: "Gunnar Gotshalks"
	date: "2003 July 16"
	version: "1.0"

class ROOT_CLASS

create make

--------------------------------------------------------------------------------
feature {NONE} -- Creation

a : ARRAY[INTEGER]

make
  -- Creation procedure.
  local n, i, size  : INTEGER
        r : TUPLE [STRING, INTEGER, ARRAY[INTEGER]]
  do
    io.new_line ; io.put_string("Start test")

    n := 5
    io.new_line ; io.put_string("Sum of integers to n = ")
      io.put_integer(n) ; io.put_string(" is ")
      io.put_integer(sum_to_i(n))

    io.new_line ; io.put_string("   Agent sum of integers to n = ")
      io.put_integer(n) ; io.put_string(" is ")
      io.put_integer(sum_to(n, agent identity(?)))
      io.new_line

    io.new_line ; io.put_string("Sum double of integers to n = ")
      io.put_integer(n) ; io.put_string(" is ")
      io.put_integer(sum_to_double_i(n))

    io.new_line ; io.put_string("   Agent sum double of integers to n = ")
      io.put_integer(n) ; io.put_string(" is ")
      io.put_integer(sum_to(n, agent double(?)))
      io.new_line

    io.new_line ; io.put_string("Sum square of integers to n = ")
      io.put_integer(n) ; io.put_string(" is ")
      io.put_integer(sum_to_square_i(n))

    io.new_line ; io.put_string("   Agent sum square of integers to n = ")
      io.put_integer(n) ; io.put_string(" is ")
      io.put_integer(sum_to(n, agent square(?)))
    io.new_line ; io.new_line

-- Create an array with five elements

    n := 5
    create a.make_filled(0, 1, n)
    print("Initial array values%N")
    from i := a.lower until i > a.upper
    loop a.put(10*i, i) ; io.put_integer(a[i]) ; io.put_character(' ')
         i := i + 1
    end

-- Use allSatisfy function

    size := 3
    io.new_line ; io.put_string("All satisfy " + size.out
                               + " is less than: ")
      io.put_boolean(all_satisfy(size, a, agent less_than(?,?)))

-- Use curried allSatisfy1
    io.new_line ; io.put_string("Curried all satisfy1 " + size.out
                               + " is less than: ")
      io.put_boolean(all_satisfy_1(a, agent less_than(size,?)))

-- Modify all array elements

    io.new_line ; io.new_line ;
    io.put_string("5 is added to each array element by an agent%N")
    modifyall(a, agent modifier_1(?))

    from i := a.lower until i > a.upper
    loop  io.put_integer(a[i]) ; io.put_string("  ") ; i := i + 1
    end

-- Tuple contents is accessed as arrays with index beginning at 1.

    r := tuple_result
    if attached {STRING} r[1] as tuple_str then
      io.put_string ("%N%NTuple string is %"" + tuple_str + "%"%N")
    end
    if attached {INTEGER_32_REF} r[2] as tuple_int then
      io.put_string ("Tuple integer is " + tuple_int.out + "%N")
    end
    if attached {ARRAY[INTEGER]} r[3] as tuple_arr then
      io.put_string ( "Tuple array bounds are " + tuple_arr.lower.out
                    + " " + tuple_arr.upper.out + "%N")
    end

    io.put_string("%NEnd test%N")
  end

--------------------------------------------------------------------------------
feature -- summation

sum_to_i (n : INTEGER) : INTEGER
  -- Sum i for i = 1..n
  local sum : INTEGER
        i : INTEGER
  do
  	from i := 1
  	until i > n
  	loop
      sum := sum + i
  	  i := i + 1
  	end
  	Result := sum
  end

sum_to_double_i (n : INTEGER) : INTEGER
  -- Sum i+i for i = 1..n
  local sum : INTEGER
        i : INTEGER
  do
  	from i := 1
  	until i > n
  	loop
      sum := sum + i + i
  	  i := i + 1
  	end
  	Result := sum
  end

sum_to_square_i (n : INTEGER) : INTEGER
  -- Sum i*i for i = 1..n
  local sum : INTEGER
        i : INTEGER
  do
  	from i := 1
  	until i > n
  	loop
      sum := sum + i*i
  	  i := i + 1
  	end
  	Result := sum
  end

sum_to (n : INTEGER ; f : FUNCTION[ANY, TUPLE[INTEGER], INTEGER]) : INTEGER
  -- Sum integer function f(i) for i = 1..n
  local sum : INTEGER
        i : INTEGER
  do
  	from i := 1
  	until i > n
  	loop
      sum := sum + f.item([i])
  	  i := i + 1
  	end
  	Result := sum
  end

--------------------------------------------------------------------------------
feature -- Function agents

all_satisfy ( size : INTEGER ; aValue : ARRAY[INTEGER]
            ; comparator : FUNCTION[ANY, TUPLE[INTEGER, INTEGER], BOOLEAN]
            ) : BOOLEAN
    --  Result is true if all items in aValue satisfy the comparator with
    -- respect to size
  local i : INTEGER
  do
    from i := aValue.lower ; Result := true
    until i > aValue.upper or Result = false
    loop
      Result := comparator.item([size, aValue[i]])
      i := i + 1
    end
  end

-- Curried version where
all_satisfy_1 (aValue : ARRAY[INTEGER]
              ; comparator : FUNCTION[ANY, TUPLE[INTEGER], BOOLEAN]
              ) : BOOLEAN
    -- Result is true if all items in aValue satisfy the comparator with
    -- respect to size
  local i : INTEGER
  do
    from i := aValue.lower ; Result := true
    until i > aValue.upper or Result = false
    loop
      Result := comparator.item([aValue[i]])
      i := i + 1
    end
  end

--------------------------------------------------------------------------------
feature -- Procedure agents

modifyall(aValue : ARRAY[INTEGER]
           ; a_modifier : PROCEDURE[ANY, TUPLE[INTEGER]])
  --  Modify the values according to the modifier procedure
  local i : INTEGER
  do
    from i := aValue.lower
    until i > aValue.upper
    loop
      a_modifier.call([i])
      i := i + 1
    end
  end

--------------------------------------------------------------------------------
feature -- Agents

identity(i : INTEGER) : INTEGER
  -- Return the argument
  do Result := i end

double(i : INTEGER) : INTEGER
  -- Return double the argument
  do Result := i+i end

square(i : INTEGER) : INTEGER
  -- Return square the argument
  do Result := i*i end

less_than(i, j : INTEGER) : BOOLEAN
  -- Result is i < j
  do
  	Result := i < j
  end

modifier_1 ( index : INTEGER )
  -- Add 5 to a[index]
  do
    a.put(a[index] + 5, index)
  end


--------------------------------------------------------------------------------
feature -- Tuples

tuple_result : TUPLE [STRING, INTEGER, ARRAY[INTEGER]]
  local a2 : ARRAY[INTEGER]
        j : INTEGER
  do
    create a2.make_filled (0, -3, +3)
    from j := -4
    until j = +3
    loop j := j + 1 ; a2[j] := j*10
    end
    Result := [ "This is the string returned by tuple."
              , 10
              , a2]
  end

end
